@@ -161,14 +161,14 @@ class Test_EUI(unittest.TestCase):
         self.failUnless(str(mac) == '00-C0-29-C2-52-FF')
         self.failUnless(mac.oui() == '00-C0-29')
         self.failUnless(mac.ei() == 'C2-52-FF')
-        self.failUnless(mac.to_eui64() == EUI('00-C0-29-FF-FE-C2-52-FF'))
+        self.failUnless(mac.eui64() == EUI('00-C0-29-FF-FE-C2-52-FF'))
 
     def testEUI64(self):
         eui64 = EUI('00-C0-29-FF-FE-C2-52-FF')
         self.failUnless(str(eui64) == '00-C0-29-FF-FE-C2-52-FF')
         self.failUnless(eui64.oui() == '00-C0-29')
         self.failUnless(eui64.ei() == 'FF-FE-C2-52-FF')
-        self.failUnless(eui64.to_eui64() == EUI('00-C0-29-FF-FE-C2-52-FF'))
+        self.failUnless(eui64.eui64() == EUI('00-C0-29-FF-FE-C2-52-FF'))
 
     def testIPv6LinkLocal(self):
         expected = 'fe80::20f:1fff:fe12:e733'
@@ -333,7 +333,7 @@ class Test_Addr_IPv6(unittest.TestCase):
         r2 = CIDR('2001:0DB8:0:CD30::/60')
         #DEBUG: print r2
 
-        r3 = CIDR('2001:0DB8::CD3/60')
+        r3 = IP('2001:0DB8::CD3/60')
         #DEBUG: print r3
 
         self.failUnless(r1 == r2)
@@ -505,7 +505,7 @@ class Test_Xrange_Generators(unittest.TestCase):
 #-----------------------------------------------------------------------------
 class Test_CIDR(unittest.TestCase):
     """
-    Tests for the CIDR class.
+    Tests for the CIDR aggregate class.
     """
     def test_IPv4_CIDR_Equality(self):
         cidr1 = CIDR('192.168.0.0/255.255.254.0')
@@ -533,7 +533,7 @@ class Test_CIDR(unittest.TestCase):
         ]
 
         supernet = CIDR('10.0.0.0/24')
-        subnets = list(str(addr) for addr in supernet)[::16]
+        subnets = list(addr for addr in supernet)[::16]
 
         for i, subnet_addr in enumerate(subnets):
             subnet = CIDR("%s/28" % subnet_addr)
@@ -550,7 +550,7 @@ class Test_CIDR(unittest.TestCase):
 
     def testIndexingAndSlicing(self):
         #   IPv4
-        c1 = CIDR('192.168.0.1/23', klass=str)
+        c1 = CIDR('192.168.0.0/23', klass=str)
 
         #   Handy methods.
         self.failUnless(c1.first() == '192.168.0.0')
@@ -600,6 +600,153 @@ class Test_CIDR(unittest.TestCase):
         self.failIf(c1 != c2)
         self.failIf(c1 is c2)
 
+    def test_CIDR_abbreviations(self):
+        abbreviations = (
+            #   Integer values.
+            (-1, None),
+            (0,   '0.0.0.0/8'),       #   Class A
+            (10,  '10.0.0.0/8'),
+            (127, '127.0.0.0/8'),
+            (128, '128.0.0.0/16'),    #   Class B
+            (191, '191.0.0.0/16'),
+            (192, '192.0.0.0/24'),    #   Class C
+            (223, '223.0.0.0/24'),
+            (224, '224.0.0.0/8'),     #   Class D (multicast)
+            (239, '239.0.0.0/8'),
+            (240, '240.0.0.0/32'),    #   Class E (reserved)
+            (255, '255.0.0.0/32'),
+            (256, None),
+            #   String values.
+            ('-1', None),
+            ('0',   '0.0.0.0/8'),       #   Class A
+            ('10',  '10.0.0.0/8'),
+            ('127', '127.0.0.0/8'),
+            ('128', '128.0.0.0/16'),    #   Class B
+            ('191', '191.0.0.0/16'),
+            ('192', '192.0.0.0/24'),    #   Class C
+            ('223', '223.0.0.0/24'),
+            ('224', '224.0.0.0/8'),     #   Class D (multicast)
+            ('239', '239.0.0.0/8'),
+            ('240', '240.0.0.0/32'),    #   Class E (reserved)
+            ('255', '255.0.0.0/32'),
+            ('256', None),
+            ('128/8',       '128.0.0.0/8'),
+            ('128.0/8',     '128.0.0.0/8'),
+            ('128.0.0.0/8', '128.0.0.0/8'),
+            ('128.0.0/8',   '128.0.0.0/8'),
+            ('192.168',     '192.168.0.0/24'),
+#FIXME:     ('192.168/8',   '192.168.0.0/8'), # Invalid with strict checking!
+            ('0.0.0.0',     '0.0.0.0/8'),
+            ('::/128',      None),            #   Does not support IPv6.
+            ('::10/128',    None),            #   Hmmm... ambiguous IPv6.
+            ('::/128',      None),            #   Does not support IPv6.
+#FIXME:            ('::192.168',   '::192.168.0.0/128'),
+#FIXME:            ('::192.168',   '::192.168.0.0/128'),
+#FIXME:            ('::ffff:192.168/120', '::ffff:192.168.0.0/120'),
+            ('0.0.0.0.0', None),
+            ('', None),
+            (None, None),
+            ([], None),
+            ({}, None),
+        )
+
+        for (abbrev, expected) in abbreviations:
+            result = abbrev_to_cidr(abbrev)
+            self.failUnless(result == expected, "expected %r, result %r" \
+                % (expected, result))
+            if result is not None:
+                cidr = CIDR(abbrev)
+                self.failUnless(str(cidr) == result, "expected %s, result %r" \
+                    % (cidr, result))
+
+    def test_CIDR_to_Wildcard(self):
+        expected = (
+            ('10.0.0.1/32', '10.0.0.1', 1),
+            ('192.168.0.0/24', '192.168.0.*', 256),
+            ('172.16.0.0/12', '172.16-31.*.*', 1048576),
+            ('0.0.0.0/0', '*.*.*.*', 4294967296),
+        )
+
+        for cidr, wildcard, size in expected:
+            c1 = CIDR(cidr)
+            w1 = c1.wildcard()
+            c2 = w1.cidr()
+            self.failUnless(c1.size() == size)
+            self.failUnless(str(c1) == cidr)
+            self.failUnless(str(w1) == wildcard)
+            self.failUnless(w1.size() == size)
+            self.failUnless(c1 == c2)           #   Test __eq__()
+            self.failUnless(str(c1) == str(c2)) #   Test __str__() values too.
+
+#-----------------------------------------------------------------------------
+class Test_Wildcard(unittest.TestCase):
+    """
+    Tests for the Wildcard aggregate class.
+    """
+    def testValidWildcards(self):
+        valid_wildcards = (
+            ('192.168.0.1', 1),
+            ('0.0.0.0', 1),
+            ('255.255.255.255', 1),
+            ('192.168.0.0-31', 32),
+            ('192.168.0.*', 256),
+            ('192.168.0-1.*', 512),
+            ('192.168-169.*.*', 131072),
+            ('*.*.*.*', 4294967296),
+        )
+
+        for wildcard, expected_size in valid_wildcards:
+            wc = Wildcard(wildcard)
+            self.failUnless(wc.size() == expected_size)
+
+    def testInvalidWildcards(self):
+        invalid_wildcards = (
+            '*.*.*.*.*',
+            '*.*.*',
+            '*.*',
+            '*',
+            '192-193.*',
+            '192-193.0-1.*.*',
+            '192.168.*.0',
+            'a.b.c.*',
+            [],
+            None,
+            False,
+            True,
+        )
+        for wildcard in invalid_wildcards:
+            self.failUnlessRaises(AddrFormatError, Wildcard, wildcard)
+
+    def test_Wildcard_to_CIDR(self):
+        expected = (
+            ('10.0.0.1', '10.0.0.1/32', 1),
+            ('192.168.0.*', '192.168.0.0/24', 256),
+            ('172.16-31.*.*', '172.16.0.0/12', 1048576),
+            ('*.*.*.*', '0.0.0.0/0', 4294967296),
+        )
+
+        for wildcard, cidr, size in expected:
+            w1 = Wildcard(wildcard)
+            c1 = w1.cidr()
+            w2 = c1.wildcard()
+            self.failUnless(w1.size() == size)
+            self.failUnless(str(w1) == wildcard)
+            self.failUnless(str(c1) == cidr)
+            self.failUnless(c1.size() == size)
+            self.failUnless(w1 == w2)           #   Test __eq__()
+            self.failUnless(str(w1) == str(w2)) #   Test __str__() values too.
+
+    def test_Wilcard_without_CIDR_Equivalent(self):
+        """
+        Test valid wildcards that cannot be converted to CIDR.
+        """
+        #TODO: make this test work with failUnlessRaises() ...
+        w1 = Wildcard('10.0.0.5-6')
+        try:
+            w1.cidr()   #   This should raise an exception.
+        except AddrConversionError:
+            pass
+
 #-----------------------------------------------------------------------------
 class Test_IP_DNS(unittest.TestCase):
     def testReverseLookup_IPv4(self):
